@@ -1,45 +1,82 @@
 # coding: utf-8
 # pylint: disable=too-many-lines,too-many-public-methods
 import asyncio
-from hashlib import sha1
+import logging
+import signal
 from datetime import datetime
-from operator import attrgetter
 from functools import partialmethod
+from hashlib import sha1
 from itertools import chain
+from operator import attrgetter
 
-import msgpack
 import aioredis
+import asyncpg
+import msgpack
+import ujson as json
+from aiohttp.client_exceptions import (
+    ClientConnectionError,
+    ClientError,
+    ClientResponseError,
+)
 from first import first
 from oauthlib.oauth2.rfc6749.errors import TokenExpiredError
-
-import ujson as json
+from tenacity import (
+    after_log,
+    retry,
+    retry_if_exception,
+    stop_after_attempt,
+    wait_random_exponential,
+)
 
 from .. import config, logger
-from ..cache import Playlist, AudioFeatures, select, async_lru, db_session
-from .result import SpotifyResult
-from ..mixins import EmailMixin
+from ..cache import AudioFeatures, Playlist, async_lru, db_session, select
 from ..constants import (
     API,
     DEVICE_ID_RE,
+    MANELISTI,
     PLAYLIST_URI_RE,
+    AudioFeature,
     AuthFlow,
     TimeRange,
-    AudioFeature,
 )
 from ..exceptions import (
-    SpotifyException,
     SpotifyAuthException,
+    SpotifyDeviceUnavailableException,
+    SpotifyException,
     SpotifyForbiddenException,
     SpotifyRateLimitException,
-    SpotifyDeviceUnavailableException,
 )
+from ..mixins import EmailMixin
 from ..mixins.asynch import AuthMixin
 from ..mixins.asynch.aiohttp_oauthlib import TokenUpdated
+from .result import SpotifyResult
+
+
+def is_retryable(exc):
+    if isinstance(exc, ClientResponseError) and exc.status == 429:
+        return False
+    return isinstance(exc, (ClientError, ClientConnectionError, TokenUpdated))
+
+
+async def init_db_connection(conn):
+    await conn.set_type_codec(
+        "json", encoder=json.dumps, decoder=json.loads, schema="pg_catalog"
+    )
+    await conn.set_type_codec(
+        "jsonb", encoder=json.dumps, decoder=json.loads, schema="pg_catalog"
+    )
 
 
 class SpotifyClient(AuthMixin, EmailMixin):
-
-    def __init__(self, *args, proxy=None, requests_timeout=None, **kwargs):
+    def __init__(
+        self,
+        *args,
+        proxy=None,
+        requests_timeout=None,
+        redis=None,
+        dbpool=None,
+        **kwargs,
+    ):
         """
         Create a Spotify API object.
 
@@ -49,14 +86,15 @@ class SpotifyClient(AuthMixin, EmailMixin):
         super().__init__(*args, **kwargs)
         self.proxy = proxy
         self.requests_timeout = requests_timeout
-        self.redis = None
+        self.redis = redis
+        self._dbpool = dbpool
 
     @db_session
     def _increment_api_call_count(self):
         try:
             user = self.user
         except Exception:
-            logger.warning(f"Tried to use an inexistent user: {self.user_id}")
+            logger.warning("Tried to use an inexistent user: %s", self.user_id)
         else:
             user.api_calls += 1
             user.last_usage_at = datetime.utcnow()
@@ -66,33 +104,61 @@ class SpotifyClient(AuthMixin, EmailMixin):
             response.raise_for_status()
         except:
             exception_params = await SpotifyClient.get_exception_params(response)
-            if (
-                response.status == 429
-                or (response.status >= 500 and response.status < 600)
+            if response.status == 429 or (
+                response.status >= 500 and response.status < 600
             ):
                 raise SpotifyRateLimitException(
                     retry_after=int(response.headers.get("Retry-After", 0)),
                     **exception_params,
                 )
 
-            elif response.status == 403:
+            if response.status == 403:
                 raise SpotifyForbiddenException(**exception_params)
+            raise SpotifyException(**exception_params)
 
-            else:
-                raise SpotifyException(**exception_params)
+    @property
+    async def dbpool(self):
+        if not self._dbpool and config.database.connection.provider == "postgres":
+            db_config = {
+                k: v for k, v in config.database.connection.items() if k != "provider"
+            }
+            self._dbpool = await asyncpg.create_pool(
+                **db_config, **(config.database.pool or {}), init=init_db_connection
+            )
+            logger.info(
+                "Created DB Pool with min=%d max=%s",
+                self._dbpool._minsize,
+                self._dbpool._maxsize,
+            )
+        return self._dbpool
 
     async def ensure_redis_pool(self):
         if not self.redis:
-            self.redis = await aioredis.create_pool(
-                config.redis.url,
-                minsize=config.redis.minsize,
-                maxsize=config.redis.maxsize,
-                loop=self.loop,
+            self.redis = await aioredis.create_redis_pool(
+                (config.redis.host or "localhost", config.redis.port or 6379),
+                db=config.redis.db or 0,
+                password=config.redis.password or None,
+                ssl=config.redis.ssl or False,
+                minsize=config.redis.minsize or 1,
+                maxsize=config.redis.maxsize or 10,
+            )
+            loop = asyncio.get_event_loop()
+            loop.add_signal_handler(
+                signal.SIGTERM, lambda: asyncio.ensure_future(self.release_resources())
             )
 
-    def __del__(self):
+    async def release_resources(self):
+        if self._dbpool:
+            await self._dbpool.close()
+
         if self.redis:
-            self.redis.close()
+            try:
+                self.redis.close()
+                await self.redis.wait_closed()
+            except:
+                pass
+        if self.session:
+            await self.session.close()
 
     @staticmethod
     def _get_cache_key(url, params, payload):
@@ -110,52 +176,44 @@ class SpotifyClient(AuthMixin, EmailMixin):
     ):
         etag_key = f"{cache_key}:{config.cache.key.etag}"
         response_key = f"{cache_key}:{config.cache.key.response}"
-        logger.info(f"Cache hit: {etag_key}")
-        async with self.redis.get() as conn:
-            redis = aioredis.Redis(conn)
-            response = await redis.get(response_key)
-            tr = redis.multi_exec()
-            try:
-                results = msgpack.loads(response, encoding=config.cache.encoding)
-            except:
-                results = None
-            if not results:
-                logger.error(f"Cached response is invalid: {etag_key}")
-                tr.delete(etag_key)
-                tr.delete(response_key)
-                await tr.execute(return_exceptions=False)
-                return await self._internal_call(method, url, payload, params, headers)
-
-            tr.expire(etag_key, config.cache.expire)
-            tr.expire(response_key, config.cache.expire)
+        logger.info("Cache hit: %s", etag_key)
+        response = await self.redis.get(response_key)
+        tr = self.redis.multi_exec()
+        try:
+            results = msgpack.loads(response)
+        except:
+            results = None
+        if not results:
+            logger.error("Cached response is invalid: %s", etag_key)
+            tr.delete(etag_key)
+            tr.delete(response_key)
             await tr.execute(return_exceptions=False)
-            return SpotifyResult(results, _client=self)
+            return await self._internal_call(method, url, payload, params, headers)
+
+        tr.expire(etag_key, config.cache.expire)
+        tr.expire(response_key, config.cache.expire)
+        await tr.execute(return_exceptions=False)
+        return SpotifyResult(results, _client=self)
 
     async def _cache_response(self, etag, results, cache_key):
         if not etag:
             return
 
-        logger.debug(f"ETAG: {etag}")
+        logger.debug("ETAG: %s", etag)
         etag_key = f"{cache_key}:{config.cache.key.etag}"
         response_key = f"{cache_key}:{config.cache.key.response}"
-        async with self.redis.get() as conn:
-            redis = aioredis.Redis(conn)
-            tr = redis.multi_exec()
-            tr.setex(etag_key, config.cache.expire, etag)
-            tr.setex(
-                response_key,
-                config.cache.expire,
-                msgpack.dumps(results, encoding=config.cache.encoding),
-            )
-            await tr.execute(return_exceptions=False)
+        tr = self.redis.multi_exec()
+        tr.setex(etag_key, config.cache.expire, etag)
+        tr.setex(
+            response_key, config.cache.expire, msgpack.dumps(results),
+        )
+        await tr.execute(return_exceptions=False)
 
     async def _get_cache_header(self, cache_key):
         etag_key = f"{cache_key}:{config.cache.key.etag}"
-        async with self.redis.get() as conn:
-            redis = aioredis.Redis(conn)
-            etag = await redis.get(etag_key, encoding=config.cache.encoding)
-            if etag:
-                return {"If-None-Match": etag}
+        etag = await self.redis.get(etag_key, encoding=config.cache.encoding)
+        if etag:
+            return {"If-None-Match": etag}
 
         return {}
 
@@ -176,38 +234,57 @@ class SpotifyClient(AuthMixin, EmailMixin):
 
     @staticmethod
     async def get_exception_params(response):
+        try:
+            text = await response.text()
+        except Exception:
+            text = None
+
         return {
             "status_code": response.status,
             "url": response.url,
             "headers": response.headers,
-            "text": await response.text(),
+            "text": text,
         }
 
     # pylint: disable=too-many-locals
-
+    @retry(
+        stop=stop_after_attempt(3),
+        wait=wait_random_exponential(multiplier=1, max=10),
+        retry=retry_if_exception(is_retryable),
+        reraise=True,
+        after=after_log(logger, logging.INFO),
+    )
     async def _internal_call(
-        self, method, url, payload, params, headers=None, retries=5, check_202=False
+        self,
+        method,
+        url,
+        payload,
+        params,
+        headers=None,
+        retries=5,
+        check_202=False,
+        increment_api_calls=False,
     ):
         await self.ensure_redis_pool()
         if payload and not isinstance(payload, (bytes, str)):
             payload = json.dumps(payload)
         params = {k: v for k, v in params.items() if v is not None}
         cache_key = self._get_cache_key(url, params, payload)
-        logger.debug(f"Cache key: {cache_key}")
+        logger.debug("Cache key: %s", cache_key)
         request_args = await self._get_request_args(payload, params, headers, cache_key)
-        logger.debug(f"Request args: {json.dumps(request_args, indent=4)}")
+        logger.debug("Request args: %s", json.dumps(request_args, indent=4))
 
         try:
             req = await self.session._request(method, url, **request_args)
-        except TokenUpdated:
-            req = await self.session._request(method, url, **request_args)
-        except TokenExpiredError:
+        except TokenExpiredError as e:
+            if self.flow != AuthFlow.CLIENT_CREDENTIALS:
+                raise e
             self.user.token = None
             await self.authenticate(flow=AuthFlow.CLIENT_CREDENTIALS)
             req = await self.session._request(method, url, **request_args)
 
         async with req as resp:
-            if self.user_id:
+            if self.user_id and increment_api_calls:
                 self._increment_api_call_count()
             if resp.status == 304:
                 return await self._fetch_response_from_cache(
@@ -217,7 +294,7 @@ class SpotifyClient(AuthMixin, EmailMixin):
             if check_202 and resp.status == 202:
                 if retries > 0:
                     logger.warning(
-                        f"Device is temporarily unavailable. Retrying in 5 seconds..."
+                        "Device is temporarily unavailable. Retrying in 5 seconds..."
                     )
                     await asyncio.sleep(5)
                     return await self._internal_call(
@@ -231,7 +308,7 @@ class SpotifyClient(AuthMixin, EmailMixin):
                 await self._check_response(resp)
             except SpotifyRateLimitException as exc:
                 logger.warning(
-                    f"Reached API rate limit. Retrying in {exc.retry_after} seconds..."
+                    "Reached API rate limit. Retrying in %s seconds...", exc.retry_after
                 )
                 await asyncio.sleep(exc.retry_after)
                 return await self._internal_call(
@@ -254,8 +331,13 @@ class SpotifyClient(AuthMixin, EmailMixin):
         check_202 = kwargs.pop("check_202", False)
         if args:
             kwargs.update(args)
+
         if "device_id" in kwargs:
-            kwargs["device_id"] = await self.get_device_id(kwargs["device_id"])
+            try:
+                kwargs["device_id"] = await self.get_device_id(kwargs["device_id"])
+            except ValueError as e:
+                logger.exception(e)
+
         if not url.startswith("http"):
             url = API.PREFIX.value + url
         return await self._internal_call(
@@ -302,11 +384,18 @@ class SpotifyClient(AuthMixin, EmailMixin):
                 - tracks - a list of spotify URIs, URLs or IDs
                 - market - an ISO 3166-1 alpha-2 country code.
         """
-        track_list = map(self._get_track_id, tracks)
-        return await self._get(
-            API.TRACKS.value, ids=",".join(track_list), market=market, **kwargs
+        track_list = [self._get_track_id(t) for t in tracks]
+
+        batches = [track_list[i : i + 50] for i in range(0, len(track_list), 50)]
+        track_lists = await asyncio.gather(
+            *[
+                self._get(API.TRACKS.value, ids=",".join(t), market=market, **kwargs)
+                for t in batches
+            ]
         )
 
+        return list(chain.from_iterable(track_lists))
+
     async def artist(self, artist_id, **kwargs):
         """ returns a single artist given the artist's ID, URI or URL
 
@@ -323,8 +412,13 @@ class SpotifyClient(AuthMixin, EmailMixin):
             Parameters:
                 - artists - a list of  artist IDs, URIs or URLs
         """
-        artist_list = map(self._get_artist_id, artists)
-        return await self._get(API.ARTISTS.value, ids=",".join(artist_list), **kwargs)
+        artist_list = [self._get_artist_id(a) for a in artists]
+        batches = [artist_list[i : i + 50] for i in range(0, len(artist_list), 50)]
+        artist_lists = await asyncio.gather(
+            *[self._get(API.ARTISTS.value, ids=",".join(a), **kwargs) for a in batches]
+        )
+
+        return list(chain.from_iterable(artist_lists))
 
     async def artist_albums(
         self, artist_id, album_type=None, country=None, limit=20, offset=0, **kwargs
@@ -509,7 +603,9 @@ class SpotifyClient(AuthMixin, EmailMixin):
             **kwargs,
         )
 
-    async def user_playlist(self, user, playlist_id=None, fields=None, **kwargs):
+    async def user_playlist(
+        self, user, playlist_id=None, fields=None, market="from_token", **kwargs
+    ):
         """ Gets playlist of a user
             Parameters:
                 - user - the id of the user
@@ -524,6 +620,7 @@ class SpotifyClient(AuthMixin, EmailMixin):
         return await self._get(
             API.PLAYLIST.value.format(user_id=user, playlist_id=_id),
             fields=fields,
+            market=market,
             **kwargs,
         )
 
@@ -661,7 +758,7 @@ class SpotifyClient(AuthMixin, EmailMixin):
             )
 
         batches = [
-            {"uris": track_uris[i:i + 100]} for i in range(0, len(track_uris), 100)
+            {"uris": track_uris[i : i + 100]} for i in range(0, len(track_uris), 100)
         ]
         results = [
             self._post(
@@ -1099,6 +1196,7 @@ class SpotifyClient(AuthMixin, EmailMixin):
         seed_tracks=None,
         limit=20,
         country="from_token",
+        filter_manele=True,
         **kwargs,
     ):
         """ Get a list of recommended tracks for one to five seeds.
@@ -1135,6 +1233,22 @@ class SpotifyClient(AuthMixin, EmailMixin):
                 param = prefix + attribute.value
                 if param in kwargs:
                     params[param] = kwargs.pop(param)
+
+        if not filter_manele:
+            return await self._get(API.RECOMMENDATIONS.value, **params, **kwargs)
+
+        for _ in range(5):
+            result = await self._get(API.RECOMMENDATIONS.value, **params, **kwargs)
+            tracks = [
+                t
+                for t in result.tracks
+                if not any(
+                    "manele" in (a.genres or []) or a.id in MANELISTI for a in t.artists
+                )
+            ]
+            if tracks:
+                result.tracks = tracks
+                return result
         return await self._get(API.RECOMMENDATIONS.value, **params, **kwargs)
 
     async def recommendation_genre_seeds(self, **kwargs):
@@ -1151,7 +1265,7 @@ class SpotifyClient(AuthMixin, EmailMixin):
         # pylint: disable=no-member
         return await self._get(API.AUDIO_ANALYSIS.value.format(id=_id), **kwargs)
 
-    async def audio_features(self, track=None, tracks=None, with_cache=True, **kwargs):
+    async def audio_features(self, track=None, tracks=None, with_cache=False, **kwargs):
         """ Get audio features for one or multiple tracks based upon their Spotify IDs
             Parameters:
                 - track - a track URI, URL or ID
@@ -1164,27 +1278,38 @@ class SpotifyClient(AuthMixin, EmailMixin):
                 API.AUDIO_FEATURES_SINGLE.value.format(id=_id), **kwargs
             )
 
-        tracks = list(map(self._get_track_id, tracks or []))
+        tracks = [self._get_track_id(t) for t in tracks or []]
         cached_tracks = []
         if with_cache:
             with db_session:
                 cached_tracks = select(a for a in AudioFeatures if a.id in tracks)[:]
                 tracks = list(set(tracks) - {a.id for a in cached_tracks})
-        batches = [tracks[i:i + 100] for i in range(0, len(tracks), 100)]
+
+        batches = [tracks[i : i + 100] for i in range(0, len(tracks), 100)]
         audio_features = await asyncio.gather(
             *[
                 self._get(API.AUDIO_FEATURES_MULTIPLE.value, ids=",".join(t), **kwargs)
                 for t in batches
             ]
         )
-        with db_session:
-            new_cached_tracks = select(a for a in AudioFeatures if a.id in tracks)[:]
-            new_cached_track_ids = {a.id for a in new_cached_tracks}
-            audio_features = [
-                AudioFeatures.from_dict(t)
-                for t in chain.from_iterable(audio_features)
-                if t["id"] not in new_cached_track_ids
-            ] + cached_tracks + new_cached_tracks
+
+        if not with_cache:
+            audio_features = list(chain.from_iterable(audio_features))
+        else:
+            with db_session:
+                new_cached_tracks = select(a for a in AudioFeatures if a.id in tracks)[
+                    :
+                ]
+                new_cached_track_ids = {a.id for a in new_cached_tracks}
+                audio_features = (
+                    [
+                        AudioFeatures.from_dict(t)
+                        for t in chain.from_iterable(audio_features)
+                        if t["id"] not in new_cached_track_ids
+                    ]
+                    + cached_tracks
+                    + new_cached_tracks
+                )
         return audio_features
 
     async def devices(self, **kwargs):
@@ -1192,7 +1317,6 @@ class SpotifyClient(AuthMixin, EmailMixin):
         """
         return await self._get(API.DEVICES.value, check_202=True, **kwargs)
 
-    @async_lru(maxsize=128)
     async def get_device_id(self, device=None):
         if isinstance(device, (str, bytes)) and DEVICE_ID_RE.match(device):
             return device
