@@ -12,7 +12,6 @@ LOCAL_ATTRIBUTES = {"_client", "_next_result", "_next_result_available", "_playa
 
 
 class Playable:
-
     def __init__(self, result):
         self.result = result
         self.client = self.result._client
@@ -48,35 +47,43 @@ class Playable:
 
 
 class SpotifyResultIterator:
-
-    def __init__(self, result, limit=None):
+    def __init__(self, result, limit=None, ignore_exceptions=False):
         self.result = result
         self.limit = limit
-        self.result_iterator = iter(result)
         self.params_list = self.result.get_next_params_list(limit)
         self.requests = (
             self.result._get_with_params(params) for params in self.params_list
         )
         self.responses = limited_as_completed(
-            self.requests, config.http.concurrent_connections
+            self.requests,
+            config.http.concurrent_connections,
+            ignore_exceptions=ignore_exceptions,
         )
 
     def __aiter__(self):
-        return self
+        return self.iterate()
 
-    async def __anext__(self):
-        item = next(self.result_iterator, None)
-        if item is not None:
-            return item
+    async def iterate(self):
+        for item in self.result:
+            yield item
 
-        # pylint: disable=no-member
-        self.result_iterator = iter(await self.responses.__anext__())
-        return await self.__anext__()
+        # pylint: disable=not-an-iterable
+        async for responses in self.responses:
+            if responses is None:
+                continue
+            for response in responses:
+                yield response
 
 
 class SpotifyResult(addict.Dict):
     ITER_KEYS = (
-        "items", "artists", "tracks", "albums", "audio_features", "playlists", "devices"
+        "items",
+        "artists",
+        "tracks",
+        "albums",
+        "audio_features",
+        "playlists",
+        "devices",
     )
 
     def __init__(self, *args, **kwargs):
@@ -164,5 +171,7 @@ class SpotifyResult(addict.Dict):
 
         return None
 
-    def iterall(self, limit=None):
-        return SpotifyResultIterator(self, limit)
+    def iterall(self, limit=None, ignore_exceptions=False):
+        return SpotifyResultIterator(
+            self, limit=limit, ignore_exceptions=ignore_exceptions
+        )
