@@ -1,31 +1,36 @@
 # coding: utf-8
 # pylint: disable=too-many-lines,too-many-public-methods
-from time import sleep
 from datetime import datetime
-from operator import attrgetter
 from functools import lru_cache, partialmethod
 from itertools import chain
-
-from first import first
+from operator import attrgetter
+from time import sleep
 
 import ujson as json
+from first import first
 
 from . import logger
-from .cache import Playlist, AudioFeatures, db, select, db_session
-from .mixins import AuthMixin, EmailMixin
-from .result import SpotifyResult
-from .constants import API, DEVICE_ID_RE, PLAYLIST_URI_RE, TimeRange, AudioFeature
+from .cache import AudioFeatures, Playlist, db, db_session, select
+from .constants import (
+    API,
+    DEVICE_ID_RE,
+    MANELISTI,
+    PLAYLIST_URI_RE,
+    AudioFeature,
+    TimeRange,
+)
 from .exceptions import (
-    SpotifyException,
     SpotifyAuthException,
+    SpotifyDeviceUnavailableException,
+    SpotifyException,
     SpotifyForbiddenException,
     SpotifyRateLimitException,
-    SpotifyDeviceUnavailableException,
 )
+from .mixins import AuthMixin, EmailMixin
+from .result import SpotifyResult
 
 
 class SpotifyClient(AuthMixin, EmailMixin):
-
     def __init__(self, *args, proxies=None, requests_timeout=None, **kwargs):
         """
         Create a Spotify API object.
@@ -42,7 +47,7 @@ class SpotifyClient(AuthMixin, EmailMixin):
         try:
             user = self.user
         except Exception:
-            logger.warning(f"Tried to use an inexistent user: {self.user_id}")
+            logger.warning("Tried to use an inexistent user: %s", self.user_id)
         else:
             user.api_calls += 1
             user.last_usage_at = datetime.utcnow()
@@ -53,20 +58,18 @@ class SpotifyClient(AuthMixin, EmailMixin):
             response.raise_for_status()
         except:
             exception_params = SpotifyClient.get_exception_params(response)
-            if (
-                response.status_code == 429
-                or (response.status_code >= 500 and response.status_code < 600)
+            if response.status_code == 429 or (
+                response.status_code >= 500 and response.status_code < 600
             ):
                 raise SpotifyRateLimitException(
                     retry_after=int(response.headers.get("Retry-After", 0)),
                     **exception_params,
                 )
 
-            elif response.status_code == 403:
+            if response.status_code == 403:
                 raise SpotifyForbiddenException(**exception_params)
 
-            else:
-                raise SpotifyException(**exception_params)
+            raise SpotifyException(**exception_params)
 
     @staticmethod
     def get_exception_params(response):
@@ -95,14 +98,14 @@ class SpotifyClient(AuthMixin, EmailMixin):
             client_secret=self.client_secret,
         )
         logger.debug("HTTP Status Code: {r.status_code}")
-        logger.debug(f"{method}: {r.url}")
+        logger.debug("%s: %s", method, r.url)
         if payload and not isinstance(payload, bytes):
-            logger.debug(f"DATA: {payload}")
+            logger.debug("DATA: %s", payload)
 
         if check_202 and r.status_code == 202:
             if retries > 0:
                 logger.warning(
-                    f"Device is temporarily unavailable. Retrying in 5 seconds..."
+                    "Device is temporarily unavailable. Retrying in 5 seconds..."
                 )
                 sleep(5)
                 return self._internal_call(
@@ -116,7 +119,7 @@ class SpotifyClient(AuthMixin, EmailMixin):
             self._check_response(r)
         except SpotifyRateLimitException as exc:
             logger.warning(
-                f"Reached API rate limit. Retrying in {exc.retry_after} seconds..."
+                "Reached API rate limit. Retrying in %s seconds...", exc.retry_after
             )
             sleep(exc.retry_after)
             return self._internal_call(method, url, payload, params)
@@ -125,7 +128,7 @@ class SpotifyClient(AuthMixin, EmailMixin):
             self._increment_api_call_count()
         if r.text and r.text != "null":
             results = r.json()
-            logger.debug(f"RESP: {r.text}")
+            logger.debug("RESP: %s", r.text)
             return SpotifyResult(results, _client=self)
 
         return None
@@ -138,8 +141,13 @@ class SpotifyClient(AuthMixin, EmailMixin):
         check_202 = kwargs.pop("check_202", False)
         if args:
             kwargs.update(args)
+
         if "device_id" in kwargs:
-            kwargs["device_id"] = self.get_device_id(kwargs["device_id"])
+            try:
+                kwargs["device_id"] = self.get_device_id(kwargs["device_id"])
+            except ValueError as e:
+                logger.exception(e)
+
         if not url.startswith("http"):
             url = API.PREFIX.value + url
         return self._internal_call(
@@ -380,7 +388,9 @@ class SpotifyClient(AuthMixin, EmailMixin):
             **kwargs,
         )
 
-    def user_playlist(self, user, playlist_id=None, fields=None, **kwargs):
+    def user_playlist(
+        self, user, playlist_id=None, fields=None, market="from_token", **kwargs
+    ):
         """ Gets playlist of a user
             Parameters:
                 - user - the id of the user
@@ -395,6 +405,7 @@ class SpotifyClient(AuthMixin, EmailMixin):
         return self._get(
             API.PLAYLIST.value.format(user_id=user, playlist_id=_id),
             fields=fields,
+            market=market,
             **kwargs,
         )
 
@@ -528,7 +539,7 @@ class SpotifyClient(AuthMixin, EmailMixin):
             )
 
         batches = [
-            {"uris": track_uris[i:i + 100]} for i in range(0, len(track_uris), 100)
+            {"uris": track_uris[i : i + 100]} for i in range(0, len(track_uris), 100)
         ]
         results = [
             self._post(
@@ -945,6 +956,7 @@ class SpotifyClient(AuthMixin, EmailMixin):
             **kwargs,
         )
 
+    # pylint: disable=too-many-locals
     def recommendations(
         self,
         seed_artists=None,
@@ -952,6 +964,7 @@ class SpotifyClient(AuthMixin, EmailMixin):
         seed_tracks=None,
         limit=20,
         country="from_token",
+        filter_manele=True,
         **kwargs,
     ):
         """ Get a list of recommended tracks for one to five seeds.
@@ -988,6 +1001,22 @@ class SpotifyClient(AuthMixin, EmailMixin):
                 param = prefix + attribute.value
                 if param in kwargs:
                     params[param] = kwargs.pop(param)
+
+        if not filter_manele:
+            return self._get(API.RECOMMENDATIONS.value, **params, **kwargs)
+
+        for _ in range(5):
+            result = self._get(API.RECOMMENDATIONS.value, **params, **kwargs)
+            tracks = [
+                t
+                for t in result.tracks
+                if not any(
+                    "manele" in (a.genres or []) or a.id in MANELISTI for a in t.artists
+                )
+            ]
+            if tracks:
+                result.tracks = tracks
+                return result
         return self._get(API.RECOMMENDATIONS.value, **params, **kwargs)
 
     def recommendation_genre_seeds(self, **kwargs):
@@ -1021,7 +1050,7 @@ class SpotifyClient(AuthMixin, EmailMixin):
             with db_session:
                 cached_tracks = select(a for a in AudioFeatures if a.id in tracks)[:]
                 tracks = list(set(tracks) - {a.id for a in cached_tracks})
-        batches = [tracks[i:i + 100] for i in range(0, len(tracks), 100)]
+        batches = [tracks[i : i + 100] for i in range(0, len(tracks), 100)]
         audio_features = [
             self._get(API.AUDIO_FEATURES_MULTIPLE.value, ids=",".join(t), **kwargs)
             for t in batches
@@ -1037,7 +1066,6 @@ class SpotifyClient(AuthMixin, EmailMixin):
         """
         return self._get(API.DEVICES.value, check_202=True, **kwargs)
 
-    @lru_cache(maxsize=128)
     def get_device_id(self, device=None):
         if isinstance(device, (str, bytes)) and DEVICE_ID_RE.match(device):
             return device
