@@ -1,24 +1,23 @@
-import sys
 import random
+import sys
 import threading
 from collections import OrderedDict
 
-from first import first
 from cached_property import cached_property
+from first import first
 
 from .. import config
 from ..cache import Playlist, db_session
+from ..constants import ItemType, TimeRange, VolumeBackend
 from ..volume import (
     AlsaVolumeControl,
+    ApplescriptVolumeControl,
     LinuxVolumeControl,
     SpotifyVolumeControl,
-    ApplescriptVolumeControl,
 )
-from ..constants import ItemType, TimeRange, VolumeBackend
 
 
 class PlayerMixin:
-
     def __init__(
         self,
         *args,
@@ -185,13 +184,12 @@ class PlayerMixin:
             k[5:]: v for k, v in kwargs.items() if k.startswith("fade_")
         }
         popularity = random.choice(list(Playlist.Popularity)[:3])
-        genre = self.top_genres(
-            time_range=time_range
-        ).select().without_distinct().random(
-            1
-        )[
-            0
-        ]
+        genre = (
+            self.top_genres(time_range=time_range)
+            .select()
+            .without_distinct()
+            .random(1)[0]
+        )
         playlist = self.genre_playlist(genre.name, popularity)
         while not playlist:
             playlist = self.genre_playlist(genre.name, popularity)
