@@ -1,5 +1,5 @@
-import time
 import random
+import time
 from datetime import date, timedelta
 from itertools import chain
 
@@ -7,13 +7,12 @@ from pony.orm import get
 from pony.orm.core import CacheIndexError
 
 from .. import logger
-from ..util import normalize_features
-from ..cache import Genre, Artist, Playlist, select, db_session
+from ..cache import Artist, Genre, Playlist, db_session, select
 from ..constants import TimeRange
+from ..util import normalize_features
 
 
 class RecommenderMixin:
-
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
 
@@ -25,7 +24,7 @@ class RecommenderMixin:
             self.user_playlists("thesoundsofspotify").all(),
         )
         for playlist in results:
-            logger.info(f"Fetching {playlist.name}")
+            logger.info("Fetching %s", playlist.name)
             if playlist.id not in fetched_ids:
                 Playlist.from_dict(playlist)
             fetched_ids.add(playlist.id)
@@ -47,7 +46,9 @@ class RecommenderMixin:
                 except CacheIndexError:
                     artist = Artist[artist.id]
                 self.user.top_artists.add(artist)
-        self.user.top_genres = self.user.top_artists.genres.distinct().keys() - self.user.disliked_genres
+        self.user.top_genres = (
+            self.user.top_artists.genres.distinct().keys() - self.user.disliked_genres
+        )
         if self.user.top_expires_at is None:
             self.user.top_expires_at = {}
         self.user.top_expires_at[TimeRange(time_range).value] = time.mktime(
@@ -122,10 +123,11 @@ class RecommenderMixin:
         Returns:
             list: List of tracks
         """
-        artists = self.top_artists(
-            time_range=time_range
-        ).select().without_distinct().random(
-            artist_limit
+        artists = (
+            self.top_artists(time_range=time_range)
+            .select()
+            .without_distinct()
+            .random(artist_limit)
         )
         if use_related:
             artists = self.fill_with_related_artists([a.id for a in artists])
@@ -137,10 +139,9 @@ class RecommenderMixin:
 
     @db_session
     def is_disliked_artist(self, artist):
-        return (
-            artist.id in set(self.user.disliked_artists.id.distinct().keys())
-            or bool(set(artist.genres or []) & set(self.user.disliked_genres))
-        )
+        return artist.id in set(
+            self.user.disliked_artists.id.distinct().keys()
+        ) or bool(set(artist.genres or []) & set(self.user.disliked_genres))
 
     @db_session
     def is_not_disliked_artist(self, artist):
