@@ -1,4 +1,6 @@
+import inspect
 from typing import Dict, List
+from itertools import chain, repeat
 
 import pandas as pd
 
@@ -36,3 +38,12 @@ def normalize_features(
     if AudioFeature.POPULARITY.value in data:
         data.popularity /= MAX_POPULARITY
     return data
+
+
+def ncycles(iterable, n):
+    "Returns the sequence elements n times"
+    return chain.from_iterable(repeat(tuple(iterable), n))
+
+
+def function_trace():
+    return " -> ".join(f.function for f in reversed(inspect.stack()[1:]))
